(******************************************************************************)
(* Fusion Folder Keeper - Transpear Software                                  *)
(******************************************************************************)
{                                                                              }
{ VERSION      : 1.2                                                           }
{ AUTHOR       : Kelvin Westlake                                               }
{ CREATED      : 22 Semptember 2000                                            }
{ WEBSITE      : http://www.Transpear.cjb.net                                  }
{ EMAIL        : KWestlae@yahoo.com                                            }
{                                                                              }
{ NOTES:                                                                       }
{ DirListClass is used for converting the information in MenuItemsRec into a   }
{ Class so that it can be stored in a list, to some people this might seem     }
{ silly but in my experience this is much simpler that other methods.          }
{                                                                              }
{ Writing editors has always made me appreciatte how difficult it is to make a }
{ code thats error resistant, this is almost an impossible task. but trying to }
{ Counter measure mistakes that users may make and also getting the program to }
{ stay within parameters without restricting it much is even more difficult.   }
{******************************************************************************}

unit Editor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ImgList, Menus,FileCtrl, ComCtrls;

Type MenuPosType = (ttfLeft, ttfRight, ttfMerged);

Type OptionsRec = Record
       AutoStart : Boolean;
       Desktop   : Boolean;
       MenuPos   : MenuPosType;
     End;


  Type MenuItemsRec = Record
       Name      : String [30];
       Path      : String [255];
       IconPath  : String [255];
       IconIndex : Integer;
     End;

Type DirListClass = Class
       Name      : String [30];
       Path      : String [255];
       IconPath  : String [255];
       IconIndex : Integer;
      End;


type
  TTFKeeperEditor = class(TForm)
    Panel1: TPanel;
    OKBut: TButton;
    CancelBut: TButton;
    IconList: TImageList;
    PageControl: TPageControl;
    EditSheet: TTabSheet;
    Panel2: TPanel;
    AddFolderButton: TSpeedButton;
    InsertFolderButton: TSpeedButton;
    DeleteFolderButton: TSpeedButton;
    DirListBox: TListBox;
    Panel3: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    IconSelectBut: TSpeedButton;
    FolderSelect: TSpeedButton;
    StaticText1: TStaticText;
    IconEdit: TEdit;
    FolderEdit: TEdit;
    NameEdit: TEdit;
    OptionsSheet: TTabSheet;
    RadioGroup1: TRadioGroup;
    LeftMenuOption: TRadioButton;
    RightMenuOption: TRadioButton;
    MergeMenuOption: TRadioButton;
    RadioGroup2: TRadioGroup;
    AutoStartOption: TCheckBox;
    DesktopOption: TCheckBox;
    procedure DirListBoxClick(Sender: TObject);
    procedure IconSelectButClick(Sender: TObject);
    procedure DeleteFolderButtonClick(Sender: TObject);
    procedure FolderSelectClick(Sender: TObject);
    procedure AddFolderButtonClick(Sender: TObject);
    procedure InsertFolderButtonClick(Sender: TObject);
    procedure LeftMenuOptionClick(Sender: TObject);
    procedure RightMenuOptionClick(Sender: TObject);
    procedure MergeMenuOptionClick(Sender: TObject);
    procedure AutoStartOptionClick(Sender: TObject);
    procedure DesktopOptionClick(Sender: TObject);
  private
     DirList : TList;
     Options  : OptionsRec;
     PrevEdit : Integer;  // Store edit item so we know were to save changed to
     JustDeleting : Boolean; // used to stop edits from saving when deleting.
     function GetDir(Index: Integer): DirListClass;
     Function LoadIconToBitmap(Path : String; Index : Integer): TBitmap;
     Procedure ResetEditor;
     Procedure ReadDirListFile(Path : String);
     Procedure SaveDirListFile(Path : String);
     Procedure SaveOptions(Path : String);
     Procedure LoadOptions(Path : String);
     Procedure EditorButtonControl(CurrentState : Boolean);
   public
        Function Execute (Path : String) :   Boolean;
        Property   Dir [Index:Integer]:DirListClass READ GetDir;
   end;



implementation

Uses Misc, IconDlg, InputDialog;

{$R *.DFM}


////////////////////////////////////////////////////////////////////////////////
/// General & Misc procedures.

// Loads an Icon to a ImageList and then save it out as a Bitmap, this is
// because speedbutton Glyphs do not support .ico format
// there must be an TImageList called IconList for it to work,
// *THOUGHT* this could be changed to read straight from a Cavas.
Function TTFKeeperEditor.LoadIconToBitmap(Path : String; Index : Integer): TBitmap;
Var Temp : TIcon;
Begin
   Result:=TBitmap.create;
   Temp:=ExtractIconFromFile(Path,Index,FALSE);
   Index:=IconList.AddIcon(Temp);
   IconList.Getbitmap(Index,Result);
   LoadIconToBitmap:=Result;
   Temp.free;
End;


// Called when even the Edit boxed might need clearing, ie after deleting.
Procedure TTFKeeperEditor.ResetEditor;
Begin
  JustDeleting:=False;
  PrevEdit:=-1;
  NameEdit.text:='';
  FolderEdit.text:='';
  IconEdit.text:='';
  IconSelectBut.glyph:=nil;
End;


// Turns all Edits+Delete+Insert off when nothing is available in the folder list
Procedure TTFKeeperEditor.EditorButtonControl(CurrentState : Boolean);
Begin
  NameEdit.Enabled:=CurrentState;
  FolderEdit.Enabled:=CurrentState;
  FolderSelect.Enabled:=CurrentState;
  IconSelectBut.Enabled:=CurrentState;
  DeleteFolderButton.Enabled:=CurrentState;
  InsertFolderButton.Enabled:=CurrentState;
End;



///////////////////////////////////////////////////////////////////////////////
/// Config Load + Save routines


Procedure TTFKeeperEditor.SaveOptions(Path : String);
Var Lock : File oF OptionsRec;
Begin
  AssignFile(Lock,Path);
  Rewrite(Lock);
  Write(Lock,Options);
  CloseFile(Lock);
End;


Procedure TTFKeeperEditor.LoadOptions(Path : String);
Var Lock : File oF OptionsRec;
Begin
  AssignFile(Lock,Path);
{$I-}
  Reset(Lock);
{$I+}
  IF IOResult<>0 THEN      // If file error then we use a default config
  Begin
    Options.AutoStart:=True;          // Set default config.
    Options.MenuPos  :=ttfLeft;
    Options.Desktop  :=True;
    AutoStartOption.Checked :=True;  // Show the default config setting.
    LeftMenuOption.Checked  :=True;
    DesktopOption.Checked   :=True;
    Exit;
  End;
  Read(Lock,Options);
  DesktopOption.Checked  :=Options.Desktop;
  AutoStartOption.Checked:=Options.AutoStart;
  Case Options.MenuPos Of
       ttfLeft   : LeftMenuOption.Checked:=True;
       ttfRight  : RightMenuOption.Checked:=True;
       ttfMerged : MergeMenuOption.Checked:=true;
  End;
  CloseFile(Lock);
End;



Procedure TTFKeeperEditor.SaveDirListFile(Path : String);
Var Lock  : File oF MenuItemsRec;
    temp  : MenuItemsRec;
    Count : Integer;
Begin
  With DirListBox DO
  IF ItemIndex>-1 THEN        // Update list if needed before save
  Begin
    Dir[ItemIndex].Name:=NameEdit.Text;
    Dir[ItemIndex].Path:=FolderEdit.Text;
    DirListBox.Items.Strings[ItemIndex]:=NameEdit.Text;;
  End;
  AssignFile(Lock,Path);
  {$I-}
  Rewrite(Lock);
  {$I+}
  IF IOResult=0 THEN
  IF DirList.Count>0 THEN
  For Count:=0 To DirList.Count-1 DO
  Begin
    temp.Name:=Dir[Count].name;
    Temp.Path:=Dir[Count].Path;
    Temp.IconPath:=Dir[Count].IconPath;
    Temp.IconIndex:=Dir[Count].IconIndex;
    IF Length(temp.name)>0 THEN Write(Lock,temp);
  end
  Else MessageDlg('Unable to write data file',mtError,[mbOK],0);
  CloseFile(Lock);
End;



Procedure TTFKeeperEditor.ReadDirListFile(Path : String);
Var Lock : File Of MenuItemsRec;
    temp : MenuItemsRec;
    DirDat : DirListClass;
    Count : Integer;
Begin
  AssignFile(Lock,Path);
  {$I-}
  Reset(Lock);
  {$I+}
  IF IOResult<>0 THEN
  Begin
     {$I-}
       Rewrite(Lock);
     {$I+}
    IF IOResult<>0 THEN
    Begin
       MessageDlg('Unable to Locate the data file, an attempt was'+
                  ' made to create a one but was unsucessful',mtError,[mbOK],0);
       Halt;
    End;
  End;
  For Count:=1 To FileSize(Lock) DO
  Begin
   Read(Lock,temp);
   DirDat:=DirListClass.Create;
   DirDat.Name:=Temp.Name;
   DirDat.Path:=Temp.Path;
   DirDat.IconPath:=Temp.IconPath;
   DirDat.IconIndex:=Temp.IconIndex;
   DirList.Add(DirDat);
   DirListBox.Items.Add(DirDat.Name);
  End;
  CloseFile(Lock);
End;




////////////////////////////////////////////////////////////////////////////////
///  Folder + Icon Select button Handlers

procedure TTFKeeperEditor.FolderSelectClick(Sender: TObject);
var S : String;
begin
 IF DirListBox.ItemIndex=-1 THEN
 Begin
   ShowMessage('You must first select an Item to edit');
   Exit;
 End;
 if SelectDirectory('Select Directory', '', S) then
 Begin
   Dir[DirListBox.ItemIndex].Path:=s;
   FolderEdit.text:=s;
 End;
end;


procedure TTFKeeperEditor.IconSelectButClick(Sender: TObject);
Var IconDlg : TTFIconSelectDlg;
begin
 IF DirListBox.ItemIndex=-1 THEN
 Begin
   ShowMessage('You must first select an Item to edit');
   Exit;
 End;
 IconDlg:=TTFIconSelectDlg.create(self);
 IF IconDlg.Execute THEN
 Begin
   Dir[PrevEdit].IconPath:=IconDlg.Filename;
   Dir[PrevEdit].IconIndex:=IconDlg.IconIndex;

   IconSelectBut.Glyph:=LoadIconToBitmap(Dir[DirListBox.ItemIndex].IconPath,
                                         Dir[DirListBox.ItemIndex].IconIndex);
   IconEdit.Text:='Path: '+Dir[DirListBox.ItemIndex].IconPath+',Index: '+
                            IntToStr(Dir[DirListBox.ItemIndex].IconIndex);
  End;
 IconDlg.free;
End;




////////////////////////////////////////////////////////////////////////////////
/// Add, Insert, Delete button Handlers

// The only way the program knows to save what a user has typed in the
// edit is through clicking OK or selecting a different item in the Listbox
// we must ensure that the list box OnClick handler has a way of knowing that
// we are deleting and not to update the cotents of our folder list this
// is the purpose of the JustDeleting var.

procedure TTFKeeperEditor.DeleteFolderButtonClick(Sender: TObject);
begin
   IF (DirListBox.Items.Count>=1) AND (DirListBox.ItemIndex>=0) THEN
   Begin
    InsertFolderButton.Enabled:=True;
    JustDeleting:=True;
    DirList.Delete(DirListBox.ItemIndex);
    DirListBox.Items.Delete(DirListBox.ItemIndex);
    ResetEditor;
 End;
 IF (DirListBox.Items.Count=0) AND (DirListBox.ItemIndex=-1) THEN
  EditorButtonControl(False);  // If no more items then disable Delete etc.
end;


// Add a New Folder handle
procedure TTFKeeperEditor.AddFolderButtonClick(Sender: TObject);
Var InputDlg : TTFinputDlg;
    temp     : DirListClass;
Begin
  InputDlg:=TTFinputDlg.Create(Self);
  IF InputDlg.Execute THEN
  Begin
    ResetEditor;
    Temp:=DirListClass.Create;
    Temp.Name:=InputDlg.Input;
    Temp.Path:='';                 // Create default setting.
    Temp.IconPath:='NONE';         // use default folder Icon.
    TEMP.IconIndex:=0;
    DirList.Add(Temp);
    DirListBox.Items.Add(InputDlg.Input);
  End;
   InputDlg.free;
   IF DirListBox.Items.Count>=1
            THEN EditorButtonControl(True);  // re-enable controls as item exist
End;

// Insert a New Folder handle
procedure TTFKeeperEditor.InsertFolderButtonClick(Sender: TObject);
Var InputDlg : TTFinputDlg;
    temp     : DirListClass;
Begin
 IF DirListBox.ItemIndex=-1 THEN
 Begin
   ShowMessage('You must select a position for the new Folder before it can inserted');
   Exit;
 End;
 InputDlg:=TTFinputDlg.Create(Self);
 IF InputDlg.Execute THEN
  Begin
    Temp:=DirListClass.Create;
    Temp.Name:=InputDlg.Input;
    Temp.Path:='';
    Temp.IconPath:='NONE';
    TEMP.IconIndex:=0;
    DirList.Insert(DirListBox.ItemIndex,Temp);
    DirListBox.Items.Insert(DirListBox.ItemIndex,Temp.name);
    ResetEditor;
   End;
  InputDlg.free;
End;




////////////////////////////////////////////////////////////////////////////////
/// Selection / Input handling routines.

procedure TTFKeeperEditor.DirListBoxClick(Sender: TObject);
begin

  IF (PrevEdit>-1) AND (Not JustDeleting) THEN // update if contents exist and
  Begin                                        // we are not deleting.
    Dir[PrevEdit].Name:=NameEdit.Text;         // because ItemIndex has now
    Dir[PrevEdit].Path:=FolderEdit.Text;       // changed we used PrevEdit.
    DirListBox.Items.Strings[PrevEdit]:=NameEdit.Text;;
  End;

  NameEdit.Text:=Dir[DirListBox.ItemIndex].Name;
  FolderEdit.Text:=Dir[DirListBox.ItemIndex].Path;

  IconEdit.Text:='Path: '+Dir[DirListBox.ItemIndex].IconPath+',Index: '+
                  IntToStr(Dir[DirListBox.ItemIndex].IconIndex);

  IconSelectBut.Glyph:=LoadIconToBitmap(Dir[DirListBox.ItemIndex].IconPath,
                                        Dir[DirListBox.ItemIndex].IconIndex);

  PrevEdit:=DirListBox.ItemIndex;    // Store current edit item place.
  NameEdit.Enabled:=True;
  FoldeRedit.Enabled:=True;
  IconEdit.Enabled:=True;
  IconSelectBut.Enabled:=True;

  IF JustDeleting THEN JustDeleting:=False;  // Reset for next use.

end;



////////////////////////////////////////////////////////////////////////////////
/// Options radio button Handlers

procedure TTFKeeperEditor.LeftMenuOptionClick(Sender: TObject);
begin
 Options.MenuPos:=ttfLeft;
end;

procedure TTFKeeperEditor.RightMenuOptionClick(Sender: TObject);
begin
  Options.MenuPos:=ttfRight;
end;

procedure TTFKeeperEditor.MergeMenuOptionClick(Sender: TObject);
begin
 Options.MenuPos:=ttfMerged;
end;

procedure TTFKeeperEditor.AutoStartOptionClick(Sender: TObject);
begin
  Options.AutoStart:=AutoStartOption.Checked;
end;


procedure TTFKeeperEditor.DesktopOptionClick(Sender: TObject);
begin
  Options.Desktop:=DesktopOption.Checked;
end;


////////////////////////////////////////////////////////////////////////////////
//// Main Execute + Property calls


// An easier way to access our folder information (used for properties)
function TTFKeeperEditor.GetDir(Index: Integer): DirListClass;
begin
  Result := DirListClass(DirList[Index]);
end;


Function TTFKeeperEditor.Execute ( Path : String): Boolean;
Begin
  PrevEdit:=-1;                      // No Items selected.
  justDeleting:=False;
  DirList:=TList.create;
  ReadDirListFile(Path+'\Paths.dat');
  LoadOptions(Path+'\Options.dat');
  PageControl.ActivePage:=EditSheet;
  if ShowModal=mrOK Then
  Begin
    SaveDirListFile(Path+'\Paths.dat');
    SaveOptions(Path+'\Options.dat');
    Result:=True;
  end
  Else Result:=False;
  DirList.Free;
End;






end.
